@@ -17,6 +17,7 @@
 
 #include <stdbool.h> //requires VS 2015 or later
 #include "nifti1_io_core.h"
+#include "nifti1.h"
 #include <math.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -31,6 +32,7 @@
 
 #include "print.h"
 
+
 #ifndef USING_R
 void nifti_swap_8bytes( size_t n , void *ar )    // 4 bytes at a time
 {
@@ -80,6 +82,68 @@ void nifti_swap_2bytes( size_t n , void *ar )    // 2 bytes at a time
 }
 #endif
 
+/*-------------------------------------------------------------------------*/
+/*! Byte swap NIFTI-1 file header in various places and ways.
+
+    If is_nifti, swap all (even UNUSED) fields of NIfTI header.
+    Else, swap as a nifti_analyze75 struct.
+*//*---------------------------------------------------------------------- */
+void swap_nifti_header( struct nifti_1_header *h  )
+{
+
+   /* otherwise, swap all NIFTI fields */
+
+   nifti_swap_4bytes(1, &h->sizeof_hdr);
+   nifti_swap_4bytes(1, &h->extents);
+   nifti_swap_2bytes(1, &h->session_error);
+
+   nifti_swap_2bytes(8, h->dim);
+   nifti_swap_4bytes(1, &h->intent_p1);
+   nifti_swap_4bytes(1, &h->intent_p2);
+   nifti_swap_4bytes(1, &h->intent_p3);
+
+   nifti_swap_2bytes(1, &h->intent_code);
+   nifti_swap_2bytes(1, &h->datatype);
+   nifti_swap_2bytes(1, &h->bitpix);
+   nifti_swap_2bytes(1, &h->slice_start);
+
+   nifti_swap_4bytes(8, h->pixdim);
+
+   nifti_swap_4bytes(1, &h->vox_offset);
+   nifti_swap_4bytes(1, &h->scl_slope);
+   nifti_swap_4bytes(1, &h->scl_inter);
+   nifti_swap_2bytes(1, &h->slice_end);
+
+   nifti_swap_4bytes(1, &h->cal_max);
+   nifti_swap_4bytes(1, &h->cal_min);
+   nifti_swap_4bytes(1, &h->slice_duration);
+   nifti_swap_4bytes(1, &h->toffset);
+   nifti_swap_4bytes(1, &h->glmax);
+   nifti_swap_4bytes(1, &h->glmin);
+
+   nifti_swap_2bytes(1, &h->qform_code);
+   nifti_swap_2bytes(1, &h->sform_code);
+
+   nifti_swap_4bytes(1, &h->quatern_b);
+   nifti_swap_4bytes(1, &h->quatern_c);
+   nifti_swap_4bytes(1, &h->quatern_d);
+   nifti_swap_4bytes(1, &h->qoffset_x);
+   nifti_swap_4bytes(1, &h->qoffset_y);
+   nifti_swap_4bytes(1, &h->qoffset_z);
+
+   nifti_swap_4bytes(4, h->srow_x);
+   nifti_swap_4bytes(4, h->srow_y);
+   nifti_swap_4bytes(4, h->srow_z);
+
+   return ;
+}
+
+bool littleEndianPlatform ()
+{
+    uint32_t value = 1;
+    return (*((char *) &value) == 1);
+}
+
 int isSameFloat (float a, float b) {
     return (fabs (a - b) <= FLT_EPSILON);
 }
@@ -498,6 +562,288 @@ mat44 nifti_mat44_inverse( mat44 R )
 }
 #endif
 
+// Eigen decomposition for symmetric 3x3 matrices, port of public domain Java Matrix library JAMA.
+// Connelly Barnes http://barnesc.blogspot.com/2007/02/eigenvectors-of-3x3-symmetric-matrix.html
+// see also https://www.geometrictools.com/Documentation/RobustEigenSymmetric3x3.pdf
+//begin: Connelly Barnes code
+#include <math.h>
+
+#ifdef MAX
+#undef MAX
+#endif
+
+#define MAX(a, b) ((a)>(b)?(a):(b))
+
+#define n 3
+
+static double hypot2(double x, double y) {
+  return sqrt(x*x+y*y);
+}
+
+// Symmetric Householder reduction to tridiagonal form.
+
+static void tred2(double V[n][n], double d[n], double e[n]) {
+//  This is derived from the Algol procedures tred2 by
+//  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
+//  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
+//  Fortran subroutine in EISPACK.
+  for (int j = 0; j < n; j++) {
+    d[j] = V[n-1][j];
+  }
+  // Householder reduction to tridiagonal form.
+  for (int i = n-1; i > 0; i--) {
+    // Scale to avoid under/overflow.
+    double scale = 0.0;
+    double h = 0.0;
+    for (int k = 0; k < i; k++) {
+      scale = scale + fabs(d[k]);
+    }
+    if (scale == 0.0) {
+      e[i] = d[i-1];
+      for (int j = 0; j < i; j++) {
+        d[j] = V[i-1][j];
+        V[i][j] = 0.0;
+        V[j][i] = 0.0;
+      }
+    } else {
+      // Generate Householder vector.
+      for (int k = 0; k < i; k++) {
+        d[k] /= scale;
+        h += d[k] * d[k];
+      }
+      double f = d[i-1];
+      double g = sqrt(h);
+      if (f > 0) {
+        g = -g;
+      }
+      e[i] = scale * g;
+      h = h - f * g;
+      d[i-1] = f - g;
+      for (int j = 0; j < i; j++) {
+        e[j] = 0.0;
+      }
+      // Apply similarity transformation to remaining columns.
+      for (int j = 0; j < i; j++) {
+        f = d[j];
+        V[j][i] = f;
+        g = e[j] + V[j][j] * f;
+        for (int k = j+1; k <= i-1; k++) {
+          g += V[k][j] * d[k];
+          e[k] += V[k][j] * f;
+        }
+        e[j] = g;
+      }
+      f = 0.0;
+      for (int j = 0; j < i; j++) {
+        e[j] /= h;
+        f += e[j] * d[j];
+      }
+      double hh = f / (h + h);
+      for (int j = 0; j < i; j++) {
+        e[j] -= hh * d[j];
+      }
+      for (int j = 0; j < i; j++) {
+        f = d[j];
+        g = e[j];
+        for (int k = j; k <= i-1; k++) {
+          V[k][j] -= (f * e[k] + g * d[k]);
+        }
+        d[j] = V[i-1][j];
+        V[i][j] = 0.0;
+      }
+    }
+    d[i] = h;
+  }
+  // Accumulate transformations.
+  for (int i = 0; i < n-1; i++) {
+    V[n-1][i] = V[i][i];
+    V[i][i] = 1.0;
+    double h = d[i+1];
+    if (h != 0.0) {
+      for (int k = 0; k <= i; k++) {
+        d[k] = V[k][i+1] / h;
+      }
+      for (int j = 0; j <= i; j++) {
+        double g = 0.0;
+        for (int k = 0; k <= i; k++) {
+          g += V[k][i+1] * V[k][j];
+        }
+        for (int k = 0; k <= i; k++) {
+          V[k][j] -= g * d[k];
+        }
+      }
+    }
+    for (int k = 0; k <= i; k++) {
+      V[k][i+1] = 0.0;
+    }
+  }
+  for (int j = 0; j < n; j++) {
+    d[j] = V[n-1][j];
+    V[n-1][j] = 0.0;
+  }
+  V[n-1][n-1] = 1.0;
+  e[0] = 0.0;
+}
+
+// Symmetric tridiagonal QL algorithm.
+
+static void tql2(double V[n][n], double d[n], double e[n]) {
+//  This is derived from the Algol procedures tql2, by
+//  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
+//  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
+//  Fortran subroutine in EISPACK.
+  for (int i = 1; i < n; i++) {
+    e[i-1] = e[i];
+  }
+  e[n-1] = 0.0;
+  double f = 0.0;
+  double tst1 = 0.0;
+  double eps = pow(2.0,-52.0);
+  for (int l = 0; l < n; l++) {
+    // Find small subdiagonal element
+    tst1 = MAX(tst1,fabs(d[l]) + fabs(e[l]));
+    int m = l;
+    while (m < n) {
+      if (fabs(e[m]) <= eps*tst1) {
+        break;
+      }
+      m++;
+    }
+    // If m == l, d[l] is an eigenvalue,
+    // otherwise, iterate.
+    if (m > l) {
+      int iter = 0;
+      do {
+        iter = iter + 1;  // (Could check iteration count here.)
+        // Compute implicit shift
+        double g = d[l];
+        double p = (d[l+1] - g) / (2.0 * e[l]);
+        double r = hypot2(p,1.0);
+        if (p < 0) {
+          r = -r;
+        }
+        d[l] = e[l] / (p + r);
+        d[l+1] = e[l] * (p + r);
+        double dl1 = d[l+1];
+        double h = g - d[l];
+        for (int i = l+2; i < n; i++) {
+          d[i] -= h;
+        }
+        f = f + h;
+        // Implicit QL transformation.
+        p = d[m];
+        double c = 1.0;
+        double c2 = c;
+        double c3 = c;
+        double el1 = e[l+1];
+        double s = 0.0;
+        double s2 = 0.0;
+        for (int i = m-1; i >= l; i--) {
+          c3 = c2;
+          c2 = c;
+          s2 = s;
+          g = c * e[i];
+          h = c * p;
+          r = hypot2(p,e[i]);
+          e[i+1] = s * r;
+          s = e[i] / r;
+          c = p / r;
+          p = c * d[i] - s * g;
+          d[i+1] = h + s * (c * g + s * d[i]);
+          // Accumulate transformation.
+          for (int k = 0; k < n; k++) {
+            h = V[k][i+1];
+            V[k][i+1] = s * V[k][i] + c * h;
+            V[k][i] = c * V[k][i] - s * h;
+          }
+        }
+        p = -s * s2 * c3 * el1 * e[l] / dl1;
+        e[l] = s * p;
+        d[l] = c * p;
+        // Check for convergence.
+      } while (fabs(e[l]) > eps*tst1);
+    }
+    d[l] = d[l] + f;
+    e[l] = 0.0;
+  }
+  // Sort eigenvalues and corresponding vectors.
+  for (int i = 0; i < n-1; i++) {
+    int k = i;
+    double p = d[i];
+    for (int j = i+1; j < n; j++) {
+      if (d[j] < p) {
+        k = j;
+        p = d[j];
+      }
+    }
+    if (k != i) {
+      d[k] = d[i];
+      d[i] = p;
+      for (int j = 0; j < n; j++) {
+        p = V[j][i];
+        V[j][i] = V[j][k];
+        V[j][k] = p;
+      }
+    }
+  }
+}
+
+void eigen_decomposition(double A[n][n], double V[n][n], double d[n]) {
+  double e[n];
+  for (int i = 0; i < n; i++) {
+    for (int j = 0; j < n; j++) {
+      V[i][j] = A[i][j];
+    }
+  }
+  tred2(V, d, e);
+  tql2(V, d, e);
+}
+//end: Connelly Barnes code
+
+/*void printMat(char ch, double A[3][3]) {
+	printf("%c=[%g %g %g; %g %g %g; %g %g %g];\n", ch,
+		A[0][0],A[0][1],A[0][2],
+		A[1][0],A[1][1],A[1][2],
+		A[2][0],A[2][1],A[2][2]);
+}*/
+
+vec3 nifti_mat33_eig3(double bxx, double bxy, double bxz, double byy, double byz, double bzz) {
+    double A[3][3];
+    A[0][0] = bxx;
+    A[0][1] = bxy;
+    A[0][2] = bxz;
+    A[1][0] = bxy;
+    A[1][1] = byy;
+    A[1][2] = byz;
+    A[2][0] = bxz;
+    A[2][1] = byz;
+    A[2][2] = bzz;
+	double V[3][3];
+	double d[3];
+    eigen_decomposition(A,V,d);
+    //printMat('A',A);
+    //printf("[V,D] = eig(A) %%where A*V = V*D\n");
+    //printMat('V',V);
+    //printf("D = [%g 0 0; 0 %g 0; 0 0 %g]\n", d[0], d[1], d[2]);
+    vec3 v3;
+    v3.v[0] = V[0][2];
+    v3.v[1] = V[1][2];
+    v3.v[2] = V[2][2];
+	if (v3.v[0] < 0.0) {
+		//B-matrix underspecified to describe B-vector
+		// Describes direction but not polarity of B-vector
+		// e.g. We get an eigenvector, but eigenvalue can be + or -
+		//  https://github.com/rordenlab/dcm2niix/issues/265
+		// Like B2q We set the vector to have a positive x component by convention.
+		//  https://raw.githubusercontent.com/matthew-brett/nibabel/master/nibabel/nicom/dwiparams.py
+		// This ensures eddy will see this as a half shell and not attempt to interpret arbitrary signs
+		v3.v[0] = -v3.v[0];
+		v3.v[1] = -v3.v[1];
+		v3.v[2] = -v3.v[2];
+	}
+    //printf("bvec = [%g 0 0; 0 %g 0; 0 0 %g]\n", v3.v[0], v3.v[1], v3.v[2]);
+    return v3;
+}
 
 
 
